import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.swing.Icon;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import java.awt.SystemColor;
import java.awt.dnd.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;

/**
 * @author David Foster <davidfstr@gmail.com>
 */
// FIXME: extend API to allow the listening for when the file well's file is changed
// (for example, by Drag & Drop)
public class JFileWell extends JLabel {
	static Font FONT = new JLabel().getFont().deriveFont(Font.PLAIN, 10);	// 10 pt, default font face
	
	private File file;
	
	private final static int DND_ACCEPTABLE_ACTIONS = DnDConstants.ACTION_COPY;
	private Border dndAcceptBorder = BorderFactory.createLineBorder(SystemColor.textHighlight, 3);
	
	public JFileWell() {
		super.setFont(FONT);
		super.setDropTarget(new DropTarget(
			(Component) this,
			DND_ACCEPTABLE_ACTIONS,		// acceptable DnD actions
			new DropTargetListener() {
				// ### EVALUATING POTENTIAL DROPS ###
				
				public void dragEnter(DropTargetDragEvent evt) {
					if (isDragOk(evt)) {
						JFileWell.this.setBorder(dndAcceptBorder);	// show willingness to accept drop
						evt.acceptDrag(DND_ACCEPTABLE_ACTIONS);
					} else {
						evt.rejectDrag();
					}
				}
				
				public void dragOver(DropTargetDragEvent evt) {
					if (isDragOk(evt)) {
						evt.acceptDrag(DND_ACCEPTABLE_ACTIONS);
					} else {
						evt.rejectDrag();
					}
				}
				
				public void dropActionChanged(DropTargetDragEvent evt) {
					if (isDragOk(evt)) {
						evt.acceptDrag(DND_ACCEPTABLE_ACTIONS);
					} else {
						evt.rejectDrag();
					}
				}
				
				public void dragExit(DropTargetEvent evt) {
					JFileWell.this.setBorder(null);					// hide willingness to accept drop
				}
				
				private boolean isDragOk(DropTargetDragEvent evt) {
					return
						(evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) &&
						((DND_ACCEPTABLE_ACTIONS & evt.getSourceActions()) != 0);
				}
				
				// ### HANDLING DROPS ###
				
				public void drop(DropTargetDropEvent evt) {
					try {
						if ((evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) &&
							((DND_ACCEPTABLE_ACTIONS & evt.getSourceActions()) != 0))
						{
							/* Accept drop */
							boolean dropSuccessful = false;
							try {
								evt.acceptDrop(DND_ACCEPTABLE_ACTIONS);
								
								try {
									List droppedFileList = (List) evt.getTransferable().getTransferData(
										DataFlavor.javaFileListFlavor);
									if (droppedFileList.size() == 1) {
										File droppedFile = (File) droppedFileList.get(0);
										if (droppedFile.exists() && droppedFile.isFile()) {
											// FIXME: ASSUME: this method is being run on the event dispatching thread
											JFileWell.this.setFile(droppedFile);
											
											dropSuccessful = true;
										} else {
											// Cannot accept a drop of a nonexistent file or of a non-basic file
										}
									} else {
										// Cannot accept a drop of more than one file
									}
								} catch (UnsupportedFlavorException e) {
									// Cannot accept a drop if the flavor we want is unsupported
								} catch (IOException e) {
									// I/O error while accepting drop
								}
							} finally {
								// Notify DND of the success/failure of the drop operation
								evt.dropComplete(dropSuccessful);
							}
						} else {
							/* Reject drop */
							evt.rejectDrop();
						}
					} finally {
						JFileWell.this.setBorder(null);				// hide willingness to accept drop
					}
				}
			},
			true));						// accept drops
		
		this.setFile(null);
	}
	
	// ### ACCESSORS ###
	
	public void setFile(File file) {
		this.file = file;
		
		if (file == null) {
			super.setIcon(null);
			super.setText("no file selected");
		} else {
			super.setIcon(getIconForFile(file));
			super.setText(file.getName());
		}
		
		// Notify listeners that the file changed
		fireFileChanged();
	}
	
	public File getFile() {
		return this.file;
	}
	
	// ### LISTENERS ###
	
	private List fileListeners = new ArrayList();
	private final ActionEvent fileChangedEvent = new ActionEvent(this, ActionEvent.ACTION_FIRST, null);
	
	/**
	 * Adds a listener to this JFileWell that will be notified 
	 * whenever this well's file changes.
	 */
	public void addActionListener(ActionListener l) {
		fileListeners.add(l);
	}
	
	public void removeActionListener(ActionListener l) {
		fileListeners.remove(l);
	}
	
	private void fireFileChanged() {
		ActionListener[] fileListenerArray = new ActionListener[fileListeners.size()];
		fileListeners.toArray(fileListenerArray);
		
		for (int i=0, n=fileListenerArray.length; i<n; i++) {
			fileListenerArray[i].actionPerformed(fileChangedEvent);
		}
	}
	
	// ### UTILITY ###
	
	/** @return		the look-and-feel specific icon for the specified file */
	private static Icon getIconForFile(File file) {
		// XXX
		JFileChooser dummyFileChooser = new JFileChooser();
		return dummyFileChooser.getUI().getFileView(dummyFileChooser).getIcon(file);
	}
}